unit FreeOTFEfrmCDBDump;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
  PasswordRichEdit, Spin64,
  OTFEFreeOTFE_U, Buttons, SDUForms, SDUFrames,
  SDUSpin64Units, SDUFilenameEdit_U, SDUDialogs, OTFEFreeOTFE_VolumeSelect;

type
  TfrmFreeOTFECDBDump = class(TSDUForm)
    pbOK: TButton;
    pbCancel: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    lblOffset: TLabel;
    preUserKey: TPasswordRichEdit;
    seSaltLength: TSpinEdit64;
    lblSaltLengthBits: TLabel;
    lblSaltLength: TLabel;
    OpenDialog: TSDUOpenDialog;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    seKeyIterations: TSpinEdit64;
    lblKeyIterations: TLabel;
    lblOptional: TLabel;
    ckBaseIVCypherOnHashLength: TCheckBox;
    se64UnitOffset: TSDUSpin64Unit_Storage;
    feDumpFilename: TSDUFilenameEdit;
    OTFEFreeOTFEVolumeSelect1: TOTFEFreeOTFEVolumeSelect;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ControlChanged(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure OTFEFreeOTFEVolumeSelect1Change(Sender: TObject);
  private
    fLUKSDump: boolean;

    procedure SetLUKSDump(dumpLUKS: boolean);

    function GetLUKSBaseIVCypherOnHashLength(): boolean;

    function GetVolumeFilename(): string;
    function GetUserKey(): string;
    function GetOffset(): int64;
    function GetSaltLength(): integer;
    function GetKeyIterations(): integer;
    function GetDumpFilename(): string;

    procedure EnableDisableControls();
  public
    OTFEFreeOTFE: TOTFEFreeOTFE;

    // If set to TRUE, dump LUKS volume, otherwise dump FreeOTFE
    // Defaults to FreeOTFE (FALSE)
    property LUKSDump: boolean read fLUKSDump write SetLUKSDump;

    property LUKSBaseIVCypherOnHashLength: boolean read GetLUKSBaseIVCypherOnHashLength;

    property VolumeFilename: string read GetVolumeFilename;
    property UserKey: string read GetUserKey;
    property Offset: int64 read GetOffset;
    property SaltLength: integer read GetSaltLength;
    property KeyIterations: integer read GetKeyIterations;
    property DumpFilename: string read GetDumpFilename;
  end;


implementation

{$R *.DFM}

uses
  OTFEFreeOTFEBase_U,
  SDUi18n,
  SDUGeneral;


function TfrmFreeOTFECDBDump.GetVolumeFilename(): string;
begin
  Result := OTFEFreeOTFEVolumeSelect1.Filename;
end;

procedure TfrmFreeOTFECDBDump.OTFEFreeOTFEVolumeSelect1Change(Sender: TObject);
begin
{
  if not(fmeVolumeSelect.IsPartitionSelected) then
    begin
    edDumpFilename.text := VolumeFilename + '.txt';
    end;
}

  ControlChanged(Sender);
end;

function TfrmFreeOTFECDBDump.GetUserKey(): string;
begin
  Result := preUserKey.Text;
end;

function TfrmFreeOTFECDBDump.GetOffset(): int64;
begin
  Result := se64UnitOffset.Value;
end;

function TfrmFreeOTFECDBDump.GetSaltLength(): integer;
begin
  Result := seSaltLength.Value;
end;

function TfrmFreeOTFECDBDump.GetKeyIterations(): integer;
begin
  Result := seKeyIterations.Value;
end;

function TfrmFreeOTFECDBDump.GetDumpFilename(): string;
begin
  Result := feDumpFilename.Filename;
end;


procedure TfrmFreeOTFECDBDump.FormCreate(Sender: TObject);
begin
  LUKSDump := FALSE;

  preUserKey.Plaintext := TRUE;
  // FreeOTFE volumes CAN have newlines in the user's password
  preUserKey.WantReturns := TRUE;
  preUserKey.WordWrap := TRUE;
  preUserKey.Lines.Clear();
  preUserKey.PasswordChar := '*';

  // se64UnitOffset set in OnShow event (otherwise units not shown correctly)

  seSaltLength.Increment := 8;
  seSaltLength.Value := DEFAULT_SALT_LENGTH;

  seKeyIterations.MinValue := 1;
  seKeyIterations.MaxValue := 999999; // Need *some* upper value, otherwise setting MinValue won't work properly
  seKeyIterations.Increment := DEFAULT_KEY_ITERATIONS_INCREMENT;
  seKeyIterations.Value := DEFAULT_KEY_ITERATIONS;

  OTFEFreeOTFEVolumeSelect1.Filename := '';
  OTFEFreeOTFEVolumeSelect1.AllowPartitionSelect := TRUE;
  OTFEFreeOTFEVolumeSelect1.FileSelectFilter := FILE_FILTER_FLT_VOLUMESANDKEYFILES;
  OTFEFreeOTFEVolumeSelect1.FileSelectDefaultExt := FILE_FILTER_DFLT_VOLUMESANDKEYFILES;
  OTFEFreeOTFEVolumeSelect1.SelectFor := fndOpen;

  feDumpFilename.Filename := '';

  ckBaseIVCypherOnHashLength.checked := TRUE;

end;

procedure TfrmFreeOTFECDBDump.EnableDisableControls();
begin
  pbOK.Enabled := (
                   (VolumeFilename <> '') AND
                   (feDumpFilename.Filename <> '') AND
                   (KeyIterations > 0)
                  );
end;

procedure TfrmFreeOTFECDBDump.FormShow(Sender: TObject);
begin
  se64UnitOffset.Value := 0;
  OTFEFreeOTFEVolumeSelect1.OTFEFreeOTFE := OTFEFreeOTFE;

  feDumpFilename.Filter     := FILE_FILTER_FLT_TEXTFILES;
  feDumpFilename.DefaultExt := FILE_FILTER_DFLT_TEXTFILES;
  
  EnableDisableControls();

end;

procedure TfrmFreeOTFECDBDump.ControlChanged(Sender: TObject);
begin
  EnableDisableControls()
end;

procedure TfrmFreeOTFECDBDump.pbOKClick(Sender: TObject);
var
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
  startTime: TDateTime;
  stopTime: TDateTime;
  diffTime: TDateTime;
  Hour, Min, Sec, MSec: Word;
{$ENDIF}
  dumpOK: boolean;
  notepadCommandLine: string;
begin
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
  startTime := Now();
{$ENDIF}

  if LUKSDump then
    begin
    dumpOK := OTFEFreeOTFE.DumpLUKSDataToFile(
                                              VolumeFilename,
                                              UserKey,
                                              LUKSBaseIVCypherOnHashLength,
                                              DumpFilename
                                             );
    end
  else
    begin
    dumpOK := OTFEFreeOTFE.DumpCriticalDataToFile(
                           VolumeFilename,
                           Offset,
                           UserKey,
                           SaltLength,  // In bits
                           KeyIterations,
                           DumpFilename
                          );
    end;


  if dumpOK then
    begin
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
    stopTime := Now();
    diffTime := (stopTime - startTime);
    DecodeTime(diffTime, Hour, Min, Sec, MSec);
    showmessage('Time taken to dump CDB: '+inttostr(Hour)+' hours, '+inttostr(Min)+' mins, '+inttostr(Sec)+'.'+inttostr(MSec)+' secs');
{$ENDIF}

    if (SDUMessageDlg(
               _('A human readable copy of your critical data block has been written to:')+SDUCRLF+
               SDUCRLF+
               DumpFilename+SDUCRLF+
               SDUCRLF+
               _('Do you wish to open this file in Windows Notepad?'),
               mtInformation, [mbYes,mbNo], 0) = mrYes) then
      begin
      notepadCommandLine := 'notepad '+DumpFilename;

      if (WinExec(PChar(notepadCommandLine), SW_RESTORE))<31 then
        begin
        SDUMessageDlg(_('Error running Notepad'), mtError, [], 0);
        end;

      end;

    ModalResult := mrOK;
    end
  else
    begin
{$IFDEF FREEOTFE_TIME_CDB_DUMP}
    stopTime := Now();
    diffTime := (stopTime - startTime);
    DecodeTime(diffTime, Hour, Min, Sec, MSec);
    showmessage('Time taken to FAIL to dump CDB: '+inttostr(Hour)+' hours, '+inttostr(Min)+' mins, '+inttostr(Sec)+'.'+inttostr(MSec)+' secs');
{$ENDIF}

    SDUMessageDlg(
               _('Unable to dump out critical data block.')+SDUCRLF+
               SDUCRLF+
               _('Please ensure that your password and details are entered correctly, and that this file is not currently in use (e.g. mounted)'),
               mtError, [mbOK], 0);
    end;

end;


procedure TfrmFreeOTFECDBDump.SetLUKSDump(dumpLUKS: boolean);
begin
  lblOptional.Visible := dumpLUKS;
  ckBaseIVCypherOnHashLength.Visible := dumpLUKS;

  lblOffset.Enabled         := not(dumpLUKS);
  se64UnitOffset.Enabled    := not(dumpLUKS);
  lblSaltLength.Enabled     := not(dumpLUKS);
  seSaltLength.Enabled      := not(dumpLUKS);
  lblSaltLengthBits.Enabled := not(dumpLUKS);
  lblKeyIterations.Enabled  := not(dumpLUKS);
  seKeyIterations.Enabled   := not(dumpLUKS);

  if (dumpLUKS) then
    begin
    self.caption := _('Dump LUKS Details');
    se64UnitOffset.value  := 0;
    seSaltLength.value    := 0;
    seKeyIterations.value := 0;
    end
  else
    begin
    self.caption := _('Dump Critical Data Block');
    end;

  fLUKSDump := dumpLUKS;
end;

function TfrmFreeOTFECDBDump.GetLUKSBaseIVCypherOnHashLength(): boolean;
begin
  Result := ckBaseIVCypherOnHashLength.checked;
end;

END.


