unit FreeOTFEfrmInstallOnUSBDrive;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, SDUForms, ComCtrls, OTFEFreeOTFE_InstructionRichEdit;

type
  TfrmInstallOnUSBDrive = class(TSDUForm)
    pbOK: TButton;
    pbCancel: TButton;
    edPath: TEdit;
    cbDrive: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    ckSetupAutoplay: TCheckBox;
    pbBrowse: TButton;
    pbRefreshDrives: TButton;
    ckHideAutorunInf: TCheckBox;
    reInstructCopyToUSBDrive: TOTFEFreeOTFE_InstructionRichEdit;
    procedure FormShow(Sender: TObject);
    procedure pbBrowseClick(Sender: TObject);
    procedure edPathChange(Sender: TObject);
    procedure pbRefreshDrivesClick(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
  private
    procedure EnableDisableControls();

    function GetInstallDrive(): char;
    function GetInstallFullPath(): string;
    function GetInstallRelativePath(): string;

    function InstallOnUSBDrive(): boolean;
    function CreateAutorunInfFile(): boolean;
  public
    procedure PopulateUSBDrives();
  end;


implementation

{$R *.dfm}

uses
{$WARN UNIT_PLATFORM OFF}
  FileCtrl,
{$WARN UNIT_PLATFORM ON}
  SDUDialogs,
  SDUi18n,
  SDUGeneral;

{$IFDEF _NEVER_DEFINED}
// This is just a dummy const to fool dxGetText when extracting message
// information
// This const is never used; it's #ifdef'd out - SDUCRLF in the code refers to
// picks up SDUGeneral.SDUCRLF
const
  SDUCRLF = ''#13#10;
{$ENDIF}

procedure TfrmInstallOnUSBDrive.FormShow(Sender: TObject);
begin
  reInstructCopyToUSBDrive.Text :=
    _('This function provides an easy means of copying FreeOTFE to a USB drive, and configuring it to launch automatically when the USB drive is plugged in.'+SDUCRLF+
      SDUCRLF+
      'Please select the USB drive, and location on it, where you would like FreeOTFE to be copied to:');
  
  edPath.text := '\'+Application.Title;
  ckSetupAutoplay.checked := TRUE;

  PopulateUSBDrives();

  EnableDisableControls();
end;


procedure TfrmInstallOnUSBDrive.pbBrowseClick(Sender: TObject);
var
  newPath: string;
  rootPath: string;
begin
  rootPath := GetInstallDrive()+':\';
  if SelectDirectory(
                     _('Select location to copy FreeOTFE to'),
                     rootPath,
                     newPath
{$IFDEF VER185}
                     , // Comma from previous line
                     [sdNewUI, sdNewFolder]
{$ENDIF}
                    ) then
    begin
    // 3 and -2 in order to strip off the "<driveletter>:"
    edPath.text := Copy(newPath, 3, (length(newPath)-2));
    end;
    
end;

procedure TfrmInstallOnUSBDrive.pbOKClick(Sender: TObject);
begin
  if InstallOnUSBDrive() then
    begin
    SDUMessageDlg(_('FreeOTFE copy complete.'), mtInformation);
    ModalResult := mrOK;
    end;

end;

function TfrmInstallOnUSBDrive.InstallOnUSBDrive(): boolean;
var
  allOK: boolean;
  destPath: string;
  srcPath: string;
begin
  allOK := TRUE;

  destPath := GetInstallFullPath();
  srcPath := ExcludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)));

  if allOK then
    begin
    // Sanity check - user trying to install into root dir?
    // Note: GetInstallRelativePath() will return '\', at a minimum
    if (length(GetInstallRelativePath()) <= 1) then
      begin
      allOK := (SDUMessageDlg(
                       _('You have opted to copy FreeOTFE to the root directory of your USB drive, and not a subdirectory.')+SDUCRLF+
                       SDUCRLF+
                       _('Are you sure you wish to do this?'),
                       mtWarning,
                       [mbYes, mbNo],
                       0
                      ) = mrYes);
      end;
    end;

  // Copy FreeOTFE software to drive
  if allOK then
    begin                  
    // CopyFile(...), but using Windows API to display "flying files" dialog
    // while copying
    // Note: This force-creates the destPath directory
    if not(SDUFileCopy(srcPath+'\*', destPath)) then
      begin
      SDUMessageDlg(
                    SDUParamSubstitute(_('Unable to copy FreeOTFE to:'+SDUCRLF+
                            SDUCRLF+
                            '%1'), [destPath]),
                    mtError
                   );
      allOK := FALSE;
      end;

    end;

  // Create autorun.inf file, if needed
  if allOK then
    begin
    if ckSetupAutoplay.checked then
      begin
      if not(CreateAutorunInfFile()) then
        begin
        SDUMessageDlg(
                      _('FreeOTFE was successfully copied over, but an autoplay (autorun.inf) file could not be created.'),
                      mtWarning
                     );
        // We take this as a success - the autorun.inf is pretty minor
        allOK := TRUE;
        end;
      end;
    end;

  Result := allOK;
end;

function TfrmInstallOnUSBDrive.CreateAutorunInfFile(): boolean;
var
  autorunContent: TStringList;
  allOK: boolean;
  partPath: string;
  srcExeFilename: string;
  autorunFilename: string;
begin
  allOK := FALSE;

  autorunContent := TStringList.Create();
  try
    partPath := GetInstallRelativePath();
    srcExeFilename := ExtractFileName(ParamStr(0));

    // Strip off any prefixing "\"
    if (length(partPath) > 0) then
      begin
      if (partPath[1] = '\') then
        begin
        partPath := Copy(partPath, 2, (length(partPath)-1));
        end;
      end;

    autorunContent.Add('[autorun]');
    autorunContent.Add('icon='+partPath+'\'+srcExeFilename);
    autorunContent.Add('open='+partPath+'\'+srcExeFilename);
    autorunContent.Add('action='+_('Launch FreeOTFE'));
    autorunContent.Add('shell\FreeOTFE='+_('Launch FreeOTFE'));
    autorunContent.Add('shell\FreeOTFE\command='+partPath+'\'+srcExeFilename);

    autorunFilename := GetInstallDrive()+':\autorun.inf';

    try
      // Try to delete any existing autorun.inf file
      if FileExists(autorunFilename) then
        begin
        DeleteFile(autorunFilename);
        end;

      autorunContent.SaveToFile(autorunFilename);

      if ckHideAutorunInf.checked then
        begin
        SetFileAttributes(PChar(autorunFilename), FILE_ATTRIBUTE_HIDDEN);
        end;

      allOK := TRUE;
    except
      on E:Exception do
        begin
        // Nothing - just swallow exception
        end;
    end;

  finally
    autorunContent.Free();
  end;

  Result := allOK;
end;

procedure TfrmInstallOnUSBDrive.pbRefreshDrivesClick(Sender: TObject);
begin
  PopulateUSBDrives();
end;

procedure TfrmInstallOnUSBDrive.PopulateUSBDrives();
begin
  SDUPopulateRemovableDrives(cbDrive);

  // Select first drive, if any available
  cbDrive.ItemIndex := -1;
  if (cbDrive.items.count > 0) then
    begin
    cbDrive.ItemIndex := 0;
    end;

end;

procedure TfrmInstallOnUSBDrive.edPathChange(Sender: TObject);
begin
  EnableDisableControls();
end;

procedure TfrmInstallOnUSBDrive.EnableDisableControls();
begin
  SDUEnableControl(cbDrive, (cbDrive.Items.count > 1));

  SDUEnableControl(ckHideAutorunInf, ckSetupAutoplay.checked);

  SDUEnableControl(
                   pbOK,
                   (
                    (cbDrive.ItemIndex >= 0) and
                    (Pos(':', edPath.text) = 0)  // No ":" allowed in path
                   )
                  );

end;

function TfrmInstallOnUSBDrive.GetInstallDrive(): char;
var
  retval: char;
begin
  retval := #0;

  if (cbDrive.ItemIndex >= 0) then
    begin
    // Only the 1st char of the drive...
    retval := cbDrive.Items[cbDrive.ItemIndex][1];
    end;

  Result := retval;
end;

function TfrmInstallOnUSBDrive.GetInstallFullPath(): string;
begin
  Result := GetInstallDrive() + ':' + GetInstallRelativePath();
end;

function TfrmInstallOnUSBDrive.GetInstallRelativePath(): string;
var
  retval: string;
begin
  retval := trim(edPath.text);
  if (Pos('\', retval) <> 1) then
    begin
    retval := '\'+retval;
    end;

  Result := retval;
end;

END.

